// 语义化版本控制模块
const semver = require('semver')
const child_process = require('child_process')
const log = require('./log');

module.exports = {
  // cmd
  cmdSync(cmd, throwError = false) {
    try {
      return child_process.execSync(cmd).toString()
    } catch (error) {
      if (throwError) console.error(error)
    }
  },
  // 检查node版本
  checkNodeVersion(wanted, id) {
    if (!semver.satisfies(process.version, wanted)) {
      log.rawBgRed(
        `You are using Node ${process.version}, but this version of ${id} requires Node ${wanted}.\nPlease upgrade your Node version.`,
      )
      process.exit(1)
    }
  },
  // commander passes the Command object itself as options,
  // extract only actual options into a fresh object.
  cleanArgs(cmd) {
    const args = {}
    cmd.options.forEach(o => {
      const key = camelize(o.long.replace(/^--/, ''))
      // if an option is not present and Command has a method with the same name
      // it should not be copied
      if (typeof cmd[key] !== 'function' && typeof cmd[key] !== 'undefined') {
        args[key] = cmd[key]
      }
    })
    return args
  },

  // 小驼峰转换
  camelize(str) {
    return str.replace(/-(\w)/g, (_, c) => (c ? c.toUpperCase() : ''))
  },
  
  // jsx模式
  get hasJsxRuntime(){
    if (process.env.DISABLE_NEW_JSX_TRANSFORM === "true") {
      return false;
    }
    try {
      require.resolve("react/jsx-runtime");
      return true;
    } catch (e) {
      return false;
    }
  },
}
