const { NODE_ENV } = process.env;
const isEnvDev = NODE_ENV === 'development';
const isEnvProd = NODE_ENV === 'production';

const lessLoader =
  (loaderOptions = {}) =>
  (config) => {
    const MiniCssExtractPlugin = require('mini-css-extract-plugin');
    const getCSSModuleLocalIdent = require('react-dev-utils/getCSSModuleLocalIdent');
    const cssLoaderOptions = loaderOptions.cssLoaderOptions || {};
    const lessLoaderOpitons = loaderOptions.lessLoaderOpitons || {};
    const styleResourcesLoaderOpitons = loaderOptions.styleResourcesLoaderOpitons || {};
    const lessRegex = /\.less$/;
    const publicPath = config.output.publicPath;
    const shouldUseRelativeAssetPaths = publicPath === './';

    function getStyleLoaders(cssOptions, preProcessors) {
      const loaders = [
        isEnvDev && require.resolve('style-loader'),
        isEnvProd && {
          loader: MiniCssExtractPlugin.loader,
          options: shouldUseRelativeAssetPaths ? { publicPath: '../../' } : {},
        },
        {
          loader: require.resolve('css-loader'),
          options: cssOptions,
        },
        {
          loader: require.resolve('postcss-loader'),
          options: {
            postcssOptions: {
              ident: 'postcss',
              config: false,
              plugins: [
                'postcss-flexbugs-fixes',
                [
                  'postcss-preset-env',
                  {
                    autoprefixer: {
                      flexbox: 'no-2009',
                    },
                    stage: 3,
                  },
                ],
                'postcss-normalize',
              ],
            },
            sourceMap: true,
          },
        },
      ].filter(Boolean);
      if (preProcessors && preProcessors.length) {
        loaders.push(
          {
            loader: require.resolve('resolve-url-loader'),
            options: {
              sourceMap: true,
            },
          },
          ...preProcessors.filter(Boolean).map((item) => ({
            loader: require.resolve(item.loader),
            options: item.options || {},
          }))
        );
      }
      return loaders;
    }

    function handleLocalIdent(context, localIdentName, localName, options) {
      const ident = getCSSModuleLocalIdent(context, localIdentName, localName, options);
      const newIdent = ident ? ident.replace(/[/+]/g, '') : '';
      return newIdent;
    }

    if (cssLoaderOptions.modules === true) {
      cssLoaderOptions.modules = {
        mode: 'local',
        getLocalIdent: handleLocalIdent,
      };
    }

    const loaders = config.module.rules.find((rule) => Array.isArray(rule.oneOf)).oneOf;
    loaders.splice(loaders.length - 1, 0, {
      test: lessRegex,
      sideEffects: true,
      use: getStyleLoaders(
        {
          importLoaders: styleResourcesLoaderOpitons ? 3 : 2,
          sourceMap: true,
          modules: {
            getLocalIdent: handleLocalIdent,
          },
          ...cssLoaderOptions,
        },
        [
          {
            loader: 'less-loader',
            options: {
              javascriptEnabled: true,
              sourceMap: true,
              ...lessLoaderOpitons,
            },
          },
          styleResourcesLoaderOpitons && {
            loader: 'style-resources-loader',
            options: {
              patterns: ['./src/styles/themes/index.less'],
              ...styleResourcesLoaderOpitons,
            },
          },
        ]
      ),
    });
    return config;
  };

module.exports = lessLoader;
